USES Crt, Dos;
{$I-}
VAR
  Adr      : Word ;
  LPTWert,
    EEType : Byte;
  DName    : String;

PROCEDURE Slow;
VAR A : LongInt;
BEGIN
  FOR A := 0 TO 10000 DO
    ASM NOP END;
END;

PROCEDURE SDA(Wert : Byte);
BEGIN
  Wert := Wert XOR 1;
  LPTWert := (LPTWert And 1) OR (Wert SHL 1);
  Port[Adr] := LPTWert OR 252;
  Slow;
END;
PROCEDURE SCK(Wert : Byte);
BEGIN
  Wert := Wert XOR 1;
  LPTWert := (LPTWert And 2) OR Wert;
  Port[Adr] := LPTWert OR 252;
  Slow;
END;
FUNCTION ISDA : Byte;
BEGIN
  ISDA := (Port[Adr+1] SHR 4) And 1;
  Slow;
END;
FUNCTION ISCK : Byte;
BEGIN
  ISCK := (Port[Adr+1] SHR 3) And 1;
  Slow;
END;


PROCEDURE GetAck;
BEGIN
  SDA(1);
  SCK(1);
  REPEAT
  UNTIL ISDA = 0;
  SCK(0);
END;
PROCEDURE Start;
BEGIN
  SCK(1);
  SDA(1);
  SDA(0);
  SCK(0);
END;
PROCEDURE Stop;
BEGIN
  SDA(0);
  SCK(1);
  SDA(1);
END;
PROCEDURE I2C_Init;
BEGIN
  SDA(1);
  SCK(0);
  STOP;
  STOP;
  STOP;
  STOP;
END;
PROCEDURE I2C_WBit(B : Byte);
BEGIN
  SDA(B);
  SCK(1);
  SCK(0);
END;
FUNCTION I2C_RBit : Byte;
BEGIN
  SDA(1);
  SCK(1);
  I2C_RBit := ISDA;
  SCK(0);
END;
PROCEDURE PutByte(Wert : Byte);
VAR
  N : Byte;
BEGIN
  FOR N := 7 DOWNTO 0 DO
    BEGIN
      I2C_WBit(Byte((Wert And 128)=128));
      Wert := Wert SHL 1;
    END;
  SDA(1);
END;
FUNCTION  GetByte : Byte;
VAR
  N, W : Byte;
BEGIN
  W := 0;
  FOR N := 7 DOWNTO 0 DO
    W := W OR (I2C_RBit SHL N);
  GetByte := W;
END;
PROCEDURE WriteByte(Bank, Adresse, Wert : Byte);
BEGIN
  Start;
  PutByte(160 Or (Bank SHL 1));
  GetAck;
  PutByte(Adresse);
  GetAck;
  PutByte(Wert);
  GetAck;
  Stop;
  REPEAT
    Start;
    PutByte(160);
  UNTIL I2C_RBit = 0; {Warten, bis Byte Programmiert}
END;
FUNCTION  ReadByte(Bank, Adresse : Byte) : Byte;
VAR
  N : Byte;
BEGIN
  Start;
  PutByte(160 OR (Bank SHL 1));
  GetAck;
  PutByte(Adresse);
  GetAck;
  Start;
  PutByte(161 OR (Bank SHL 1));
  GetAck;
  ReadByte := GetByte;
  I2C_WBit(1);
  Stop;
END;
FUNCTION  EEPromType : Byte;
VAR
  I : Byte;
BEGIN
  I := 0;
  REPEAT
    Start;
    PutByte(160 Or ((1 SHL I-1) SHL 1));
    Inc(I);
  UNTIL (I2C_RBit = 1) OR (I = 4);
  EEPromType := 1 SHL I;
END;
PROCEDURE GetInfos;
BEGIN
  I2C_Init;
  EEType := EEPromType;
  Writeln('EEProm:  24C', EEType);
  Writeln('Banks:   ', EEType SHR 1);
  Writeln('Bit:     ', 1024*EEType, ' b');
  Writeln('Byte:    ', 128*EEType, ' B = ', (128*EEType) Div 1024, ' KB');
  Writeln;
END;
PROCEDURE ReadFile;
VAR
  F     : File;
  I     : Word;
  B     : Byte;
BEGIN
  GetInfos;
  Writeln('Reading EEProm! File: "', DName, '"');
  Assign(F, DName);
  Rewrite(F, 1);
  FOR I := 0 TO (EEType SHL 7)-1 DO
    BEGIN
      B := ReadByte(I SHR 8, I And 255);
      BlockWrite(F, B, 1);
    END;
  Close(F);
  Writeln('Done!');
END;

PROCEDURE WriteFile;
VAR
  F          : File;
  I, Bank, B : Byte;
  Ps, Ln     : Word;


BEGIN
  GetInfos;
  Assign(F, DName);
  Reset(F, 1);
  IF IOResult = 0 THEN
    BEGIN
      Writeln('Programming File "', DName, '" to EEProm!');
      Ln := FileSize(F)-1;
      IF Ln > EEType*128-1 THEN
        Ln := EEType*128-1;
      FOR Ps := 0 TO Ln DO
        BEGIN
          BlockRead(F, B, 1);
          WriteByte(Ps SHR 8, Ps And 255, B);
          Write('Done: ', Ps+1, ' Byte(s), ', (Ln-Ps), ' Byte(s) to do   ');
          GotoXY(1, WhereY);
        END;
    END
  ELSE
    Writeln('File "', DName, '" does not exist');
  Close(F);
END;

BEGIN
  Adr := MemW[$40:8];
  IF (Paramcount = 0) OR ((Paramcount=1) And (ParamStr(1) = '?') OR
                                             (ParamStr(1) = '-?') OR
                                             (ParamStr(1) = 'h') OR
                                             (ParamStr(1) = '-h')) THEN
    BEGIN
      Writeln;
      Writeln('24CXX-Programmer (c) by Kai Gossner');
      Writeln;
      Writeln('INet: kai@rbks.dualedu.du.nw.schule.de');
      Writeln('FIDO: 2:2448/2425.26');
      Writeln('Options:     PROGIT [<parameter>] <File>');
      Writeln;
      Writeln('<File>       is the Filename with path of the file to programm or');
      Writeln('             to write the contents of the EEProm.');
      Writeln('<parameters> is one of the following:');
      Writeln('             r reads the contents of the EEProm and writes it as');
      Writeln('               binary to the file <File>');
      Writeln;
      Writeln('called just with the filename the file will be programmed to the EEProm');
      Writeln;
      Writeln('Examples: PROGIT PROMFILE.BIN');
      Writeln('           -> will programm the EEprom');
      Writeln('          PROGIT r PROMFILE.BIN');
      Writeln('           -> will write the contents of the EEProm to the file');
      Writeln('              PROMFILE.BIN');
      Writeln;
      Writeln('The hardware must be plugged in port LPT1 and a seriell EEProm has');
      Writeln('to be inserted.');
    END
  ELSE
    BEGIN
      LPTWert := 0;
      I2C_Init;
      DName := ParamStr(Paramcount);
      IF (Paramcount = 2) And ((ParamStr(1) = 'r') OR
                               (ParamStr(1) = 'R')) THEN
        ReadFile
      ELSE IF (Paramcount = 3)  THEN
           ELSE
             WriteFile;
    END;
  Writeln;
END.
